package com.taais.biz.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.taais.biz.constant.BizConstant;
import com.taais.biz.domain.bo.VideoStableStartResultBo;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.core.utils.MapstructUtils;
import com.taais.common.json.utils.JsonUtils;
import com.taais.common.orm.core.page.PageQuery;
import com.taais.common.core.core.page.PageResult;
import com.taais.common.orm.core.service.impl.BaseServiceImpl;
import com.taais.system.domain.vo.SysOssVo;
import com.taais.system.service.ISysOssService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.taais.biz.mapper.VideoStableMapper;
import com.taais.biz.domain.VideoStable;
import com.taais.biz.domain.bo.VideoStableBo;
import com.taais.biz.domain.vo.VideoStableVo;
import com.taais.biz.service.IVideoStableService;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import static com.taais.biz.domain.table.VideoStableTableDef.VIDEO_STABLE;

/**
 * 视频去抖动Service业务层处理
 *
 * @author 0
 * 2024-08-30
 */
@Service
@Slf4j
public class VideoStableServiceImpl extends BaseServiceImpl<VideoStableMapper, VideoStable> implements IVideoStableService {
    @Value("${server.video_stable_url}")
    private String video_stable_url;

    @Autowired
    private ISysOssService ossService;

    @Resource
    private VideoStableMapper videoStableMapper;

    @Override
    public QueryWrapper query() {
        return super.query().from(VIDEO_STABLE);
    }

    private QueryWrapper buildQueryWrapper(VideoStableBo videoStableBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(VIDEO_STABLE.NAME.like
            (videoStableBo.getName()));
        queryWrapper.and(VIDEO_STABLE.INPUT_OSS_ID.eq
            (videoStableBo.getInputOssId()));
        queryWrapper.and(VIDEO_STABLE.OUTPUT_OSS_ID.eq
            (videoStableBo.getOutputOssId()));
        queryWrapper.and(VIDEO_STABLE.START_TIME.eq
            (videoStableBo.getStartTime()));
        queryWrapper.and(VIDEO_STABLE.END_TIME.eq
            (videoStableBo.getEndTime()));
        queryWrapper.and(VIDEO_STABLE.COST_SECOND.eq
            (videoStableBo.getCostSecond()));
        queryWrapper.and(VIDEO_STABLE.LOG.eq
            (videoStableBo.getLog()));
        queryWrapper.and(VIDEO_STABLE.REMARKS.eq
            (videoStableBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询视频去抖动
     *
     * @param id 视频去抖动主键
     * @return 视频去抖动
     */
    @Override
    public VideoStableVo selectById(Long id) {
        return this.getOneAs(query().where(VIDEO_STABLE.ID.eq(id)), VideoStableVo.class);

    }

    /**
     * 查询视频去抖动列表
     *
     * @param videoStableBo 视频去抖动Bo
     * @return 视频去抖动集合
     */
    @Override
    public List<VideoStableVo> selectList(VideoStableBo videoStableBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(videoStableBo);
        return this.listAs(queryWrapper, VideoStableVo.class);
    }

    /**
     * 分页查询视频去抖动列表
     *
     * @param videoStableBo 视频去抖动Bo
     * @return 分页视频去抖动集合
     */
    @Override
    public PageResult<VideoStableVo> selectPage(VideoStableBo videoStableBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(videoStableBo);
        Page<VideoStableVo> page = this.pageAs(PageQuery.build(), queryWrapper, VideoStableVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增视频去抖动
     *
     * @param videoStableBo 视频去抖动Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(VideoStableBo videoStableBo) {
        // 检查input_oss_id是否存在
        if (ObjectUtil.isNull(videoStableBo.getInputOssId())) {
            return false;
        }

        SysOssVo ossEntity = ossService.getById(videoStableBo.getInputOssId());
        if (ObjectUtil.isNull(ossEntity)) {
            return false;
        }

        VideoStable videoStable = MapstructUtils.convert(videoStableBo, VideoStable.class);
        videoStable.setStatus(BizConstant.VideoStatus.NOT_START);

        return this.save(videoStable);// 使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 修改视频去抖动
     *
     * @param videoStableBo 视频去抖动Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(VideoStableBo videoStableBo) {
        VideoStable videoStable = MapstructUtils.convert(videoStableBo, VideoStable.class);
        if (ObjectUtil.isNotNull(videoStable) && ObjectUtil.isNotNull(videoStable.getId())) {
            boolean updated = this.updateById(videoStable);
            return updated;
        }
        return false;
    }

    /**
     * 批量删除视频去抖动
     *
     * @param ids 需要删除的视频去抖动主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

    @Override
    public CommonResult start(Long id) {
        VideoStable videoStable = getById(id);

        log.info("start video_stable_url: {}", video_stable_url);
        try (CloseableHttpClient httpClient = HttpClients.createDefault()) {
            HttpPost request = new HttpPost(video_stable_url);
            // 设置请求体
            System.out.println(JsonUtils.toJsonString(videoStable));
            StringEntity entity = new StringEntity(JsonUtils.toJsonString(videoStable), "UTF-8");
            entity.setContentType("application/json");
            request.setEntity(entity);
            CloseableHttpResponse response = httpClient.execute(request);
            String responseBody = EntityUtils.toString(response.getEntity());
            System.out.println(responseBody);

            videoStable.setStatus(BizConstant.VideoStatus.RUNNING);
            updateById(videoStable);
            return CommonResult.success("start video_stable success");
        } catch (Exception e) {
            videoStable.setStatus(BizConstant.VideoStatus.FAILED);
            updateById(videoStable);
            e.printStackTrace();
            return CommonResult.fail("start video_stable error -> " + e.getMessage());
        }
    }

    @Override
    public CommonResult getResult(VideoStableStartResultBo videoStableStartResultBo) {
        Long id = Long.valueOf(videoStableStartResultBo.getId());
        String status = videoStableStartResultBo.getStatus();
        String msg = videoStableStartResultBo.getMsg();
        VideoStable videoStable = getById(id);
        videoStable.setLog(msg);
        videoStable.setStatus("200".equals(status) ? BizConstant.VideoStatus.END : BizConstant.VideoStatus.FAILED);
        updateById(videoStable);
        return CommonResult.success();
    }
}
